<?php
namespace App\Factory;

use App\Exception;

class ConsumerFactory {
    
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getConsumer($cdcliente, $cdconsumidor) {
        $sql = "
            SELECT SGESTADO, CDMUNICIPIO, 
                   NRCEPCONS, CDBAIRRO, 
                   NMBAIRCONS, NRENDECONS, 
                   DSENDECONS, DSCOMPLENDECONS, 
                   DSREFERENDECONS
		      FROM CONSUMIDOR CO
             WHERE CDCLIENTE    = :CDCLIENTE
               AND CDCONSUMIDOR = :CDCONSUMIDOR";
		    
		$params = [
            'CDCLIENTE'     => $cdcliente,
            'CDCONSUMIDOR'  => $cdconsumidor
		];
		
		return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getNeighborhoodCode ($state, $city, $neighborhood) {
        $sql = "
            SELECT CDBAIRRO
              FROM BAIRRO
            WHERE CDPAIS          = :CDPAIS
              AND SGESTADO        = :SGESTADO
              AND CDMUNICIPIO     = :CDMUNICIPIO
              AND UPPER(NMBAIRRO) = UPPER(:NMBAIRRO)";
        
        $params = array(
            ":CDPAIS"      => "0055",
            ":SGESTADO"    => $state,
            ":CDMUNICIPIO" => $city,
            ":NMBAIRRO"    => strtoupper($neighborhood)
        );
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if (empty($result)) 
            return null;
        
        return $result[0]["CDBAIRRO"];
    }
    
    public function getCityCode ($state, $city) {
        $sql = "
            SELECT CDMUNICIPIO 
    	      FROM MUNICIPIO
    	     WHERE UPPER(SGESTADO) = :SGESTADO
               AND CDPAIS   = :CDPAIS
    		   AND NVL((SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) = :NMMUNICIPIO
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
    		             AND ROWNUM = 1),
    		         (SELECT CDMUNICIPIO
    		            FROM MUNICIPIO
    		           WHERE UPPER(NMMUNICIPIO) LIKE '%'||:NMMUNICIPIO||'%'
                         AND UPPER(SGESTADO) = :SGESTADO
                         AND CDPAIS   = :CDPAIS
    		             AND ROWNUM = 1)) = CDMUNICIPIO";
    	
    	$params = array(
    	    ":CDPAIS"      => "0055",
    	    ":SGESTADO"    => strtoupper($state),
    	    ":NMMUNICIPIO" => strtoupper($city)
    	);

    	$result = $this->connection->fetchAll($sql, $params);
    	
    	if (empty($result)) return null;
    	
    	return $result[0]["CDMUNICIPIO"];
    }
}
