<?php

namespace App\Factory;

use App\Exception\ParamsException as ParamsException;
use App\Exception\SqlException as SqlException;

class PaymentFactory {

    private $connection;
    private $util;

    public function __construct (\App\Util\Util $util,
            \Doctrine\DBAL\Connection $connection) {
        $this->util = $util;
        $this->connection = $connection;
    }

    public function getPaymentCode() {
        $sql = "
            SELECT TP.CDTIPORECE 
              FROM TIPORECEINTE TP
              JOIN INTEGRACAO IT
                ON IT.CDINTEGRACAO = TP.CDINTEGRACAO
             WHERE IT.IDINTEGRACAO = 'ANT'
        ";

        return $this->connection->fetchAssoc($sql);
    }

    public function getMoneyReceiptCode () {
        $sql = "SELECT MIN(CDTIPORECE) AS CDTIPORECE
                FROM TIPORECE
                WHERE IDTIPORECE = '4'";
        
        $result = $this->connection->fetchAll($sql);
        return $result[0]["CDTIPORECE"];
    }

    public function saveMovcaixadlv($idtipomovivedlv, $cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $paymentValue, $cdcliente, $cdconsumidor) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR)";

        $params = [
            "CDFILIAL"        => $cdfilial,
	        "NRVENDAREST"     => $nrvendarest,
	        "NRSEQMOVDLV"     => $nrseqmovdlv,
	        "IDTIPOMOVIVEDLV" => $idtipomovivedlv,
	        "CDCLIENTE"       => $cdcliente,
	        "CDCONSUMIDOR"    => $cdconsumidor,
	        "CDTIPORECE"      => $paymentCode,
	        "VRMOVIVENDDLV"   => $this->util->tofloat($paymentValue)
        ];

        return $this->connection->executeQuery($sql, $params);
    }
    
    public function getReceiptType($cdFilial, $dlvMode, $tpReceInte) {
        // $idIntegracao = ($dlvMode == "DLV_IFO") ? "IFO" : "SPO";
        $idIntegracao = $dlvMode;

        //===================== Busca por filial =====================
        $sql = "
            SELECT CDTIPORECE
              
              FROM TIPORECEINTELOCAL TP
    
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO
    
             WHERE INTE.IDINTEGRACAO = :IDINTEGRACAO
               AND CDFILIAL          = :CDFILIAL
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE    
        ";

        $params = [
            ":IDINTEGRACAO" => $idIntegracao,
            ":CDFILIAL"     => $cdFilial,
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $tpReceInte))
        ];

        $result = $this->connection->fetchAll($sql, $params);
        
        if(!empty($result) && isset($result[0]["CDTIPORECE"])) {
            return $result[0]["CDTIPORECE"];
        }
        //===================== Busca por filial =====================

        $sql = "
            SELECT CDTIPORECE

              FROM TIPORECEINTE TP
            
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO

             WHERE INTE.IDINTEGRACAO = :IDINTEGRACAO
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE
        ";
        
        $params = [
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $tpReceInte)),
            ":IDINTEGRACAO" => $idIntegracao
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if(empty($result)) 
            throw new ParamsException("
                Não existe um tipo de recebimento mapeado no sistema para o código de pagamento {$tpReceInte}.<br>
                <a href=''>Clique aqui</a> para consultar os códigos de pagamento suportados pelo Anota ai.
                Para mais detalhes sobre o mapeamento de tipos de recebimento, verifique a seção 'Parametrização da Integração', no manual da integração.");
        
        if(!isset($result[0]["CDTIPORECE"])) 
            throw new ParamsException("
                Não existe um tipo de recebimento mapeado no sistema para o código de pagamento {$tpReceInte}.<br>
                <a href=''>Clique aqui</a> para consultar os códigos de pagamento suportados pelo Anota ai.
                Para mais detalhes sobre o mapeamento de tipos de recebimento, verifique a seção 'Parametrização da Integração', no manual da integração.");
        
        return $result[0]["CDTIPORECE"];
    }
    
}
