<?php
namespace App\Service;

class ConsumerService {
    
    private $factory;
    private $util;

    public function __construct (\App\Factory\ConsumerFactory $factory, \App\Util\Util $util) {
        $this->factory = $factory;
        $this->util = $util;
    }
    
    public function getConsumer($cdcliente, $cdconsumidor) {
        $consumer = $this->factory->getConsumer($cdcliente, $cdconsumidor);
        
        if(empty($consumer)) 
            throw new \Exception("Nao foi possivel encontrar o consumidor. CDCLIENTE: $cdcliente, CDCONSUMIDOR: $cdconsumidor");
        
        return $consumer;
    }
    
    public function getConsumerAddress($address) {
        $consumerData = [
            "SGESTADO"        => "", 
            "CDMUNICIPIO"     => "", 
            "NRCEPCONS"       => "", 
            "CDBAIRRO"        => "", 
            "NMBAIRCONS"      => "", 
            "NRENDECONS"      => "", 
            "DSENDECONS"      => "", 
            "DSCOMPLENDECONS" => "", 
            "DSREFERENDECONS" => "", 
            "NMMUNICIPIO"     => ""
        ];

        $consumerData["DSCOMPLENDECONS"] = "";
        $consumerData["DSENDECONS"]      = strtoupper($this->util->removeAcentos(explode(",", $address['formatted_address'])[0]));
        $consumerData["DSREFERENDECONS"] = "";
        
        foreach($address['address_components'] as $component) {
            if(array_search("administrative_area_level_1", $component['types']) !== false) 
                $consumerData["SGESTADO"] = $component['short_name']; 

            if(array_search("postal_code", $component['types']) !== false) 
                $consumerData["NRCEPCONS"] = str_replace("-", "", $component['short_name']);

            if(array_search("street_number", $component['types']) !== false) 
                $consumerData["NRENDECONS"] = $component['short_name'];

            if(array_search("administrative_area_level_2", $component['types']) !== false) 
                $consumerData["NMMUNICIPIO"] = strtoupper($this->util->removeAcentos($component['short_name']));

            if(array_search("sublocality_level_1", $component['types']) !== false) 
                $consumerData["NMBAIRCONS"] = strtoupper($this->util->removeAcentos($component['short_name']));
        }
        
        $consumerData["CDMUNICIPIO"] = $this->factory->getCityCode($consumerData["SGESTADO"], $consumerData["NMMUNICIPIO"]);
        $consumerData["CDBAIRRO"]    = $this->factory->getNeighborhoodCode($consumerData["SGESTADO"], $consumerData["CDMUNICIPIO"], $consumerData["NMBAIRCONS"]);

        return $consumerData;
    }
}
