<?php

namespace App\Service;

use App\Exception\SqlException as SqlException;
use App\Exception\ParamsException as ParamsException;

class ProductService {
    
    private $util;
    private $factory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {
        $this->util    = $util;
        $this->factory = $factory;
    }

    public function processProduct ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ) {
       
        $processedProduct = $this->processProductAsCombo($generalParams, $product, $nrvendarest, $nrcomanda, $ITCOMANDAVEN_SEQ);

        return $processedProduct;
    }
    
    private function processProductAsCombo ($generalParams, $product, $nrvendarest, $nrcomanda, &$ITCOMANDAVEN_SEQ) {
        $productItems = [];

        if (!isset($product["externalId"]) || empty($product["externalId"]))
                throw new ParamsException("
                    O produto {$product['name']} não possui código externo e ele é obrigatório.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção na plataforma Anota Aí.<br>");
        
        $productDetails = $this->factory->getProduct($product["externalId"]);

        if (!isset($productDetails) || empty($productDetails))
                throw new ParamsException("
                    O produto {$product['name']} não possui o código externo {$product['externalId']} cadastrado no Sistema Teknisa .<br>
                    Verifique no Odhen Retail o código externo deste produto e faça as devidas correções.<br>");


        $productDetails = array_merge($productDetails, $product);
        
        $txProdComVenPai = "";
        
        for($i = 0; $i < $product["quantity"]; $i++) {
            $nrSeqProdComPai = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;

            if(!empty($productDetails["observation"])){
                $txProdComVenPai = " ".$productDetails["observation"].";";
            }
            
            
            $pruductSimpleTemplate = [
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $nrvendarest, 
                "NRCOMANDA"        => $nrcomanda,
                "NRPRODCOMVEN"     => $nrSeqProdComPai,
                "CDPRODUTO"        => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => $productDetails["price"], //bcdiv($productDetails["price"], 100, 2), 
                "TXPRODCOMVEN"     => "",
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                "VRDESCCOMVEN"     => 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => null,
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"   => null,
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => "",
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => []
            ];
            

            foreach ($product["subItems"] as $item) {
                
                if($item["price"] > 0) {
                    
                    if (!isset($item["externalId"]) || empty($item["externalId"])) 
                        throw new ParamsException("
                            O produto {$item['name']} não possui código externo e ele é obrigatório.<br>
                            Verifique no Odhen Retail o código externo deste produto e faça a correção na plataforma Anota Aí.<br>");
                    
                    $validateProd = $this->factory->getProduct($item["externalId"]);
                    
                    if (!isset($validateProd) || empty($validateProd))
                        throw new ParamsException("
                            O produto {$item['name']} não possui o código externo {$item['externalId']} cadastrado no Sistema Teknisa .<br>
                            Verifique no Odhen Retail o código externo deste produto e faça as devidas correções.<br>");

                    for($y = 0; $y < $item["quantity"]; $y++) {
                        $itemDetails = array_merge($validateProd, $item);
                        
                        $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
                        
                        $txProdComVenIt = "";
                        if(!empty($itemDetails["observation"])){
                            $txProdComVenIt = " ".$itemDetails["observation"].";";
                        }
                        
                        $itemReturn = [
                            "CDFILIAL"          => $generalParams["CDFILIAL"],
                            "NRVENDAREST"       => $nrvendarest, 
                            "NRCOMANDA"         => $nrcomanda,
                            "NRPRODCOMVEN"      => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"         => $itemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"      => 1,
                            "VRPRECCOMVEN"      => $itemDetails["price"], //bcdiv($itemDetails["price"], 100, 2),
                            "TXPRODCOMVEN"      => $txProdComVenIt,
                            "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "3",
                            "VRDESCCOMVEN"      => 0,
                            "NRLUGARMESA"       => "01",
                            "IDPRODIMPFIS"      => "N",
                            "CDLOJA"            => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"       => 0,
                            "NRSEQPRODCOM"      => $this->util->formatNrSeqProdCom($nrSeqProdComPai),
                            "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN"    => 0,
                            "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO"    => $productDetails["CDPRODUTO"],
                            "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"       => "",
                            "DSOBSPEDDIGCMD"    => null,
                            "ITEM_OBSERVATION"  => [] //
                        ];
    
                        array_push($productItems, $itemReturn);
                    }

                } 
                elseif($item["price"] == 0){

                    $validateProd = $this->factory->getObservation($item['externalId']);

                    // if(empty($pruductSimpleTemplate['TXPRODCOMVEN'])){
                    //     $pruductSimpleTemplate['TXPRODCOMVEN'] = ' ';
                    // }
                    
                    if(empty($validateProd)) {
                        $txProdComVenPai .= " ".strtoupper("{$item['name']};");
                    } else {
                        $txProdComVenPai .= " ".strtoupper("{$validateProd['DSOCORR']};");
                        array_push($pruductSimpleTemplate['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],
                            'NRVENDAREST'     => $nrvendarest,
                            'NRCOMANDA'       => $nrcomanda,
                            'NRPRODCOMVEN'    => $nrSeqProdComPai,
                            'CDGRPOCOR'       => $validateProd['CDGRPOCOR'],
                            'CDOCORR'         => $validateProd['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => $item['quantity']
                        ]);
                    }
                }
            }
            
            $pruductSimpleTemplate["TXPRODCOMVEN"] = $txProdComVenPai;
            
            array_push($productItems, $pruductSimpleTemplate);
            
        }
        return $productItems;
    }

    
    /* ----- DEPRECIADO
    public function saveObsitcomandaven($cdfilial, $nrvendarest, $nrcomanda, $formattedObservations) {
        foreach($formattedObservations as $observation) {
            $this->factory->saveObsitcomandaven($cdfilial, $nrvendarest, $nrcomanda, $observation['NRPRODCOMVEN'],
                                                $observation['CDGRPOCOR'], $observation['CDOCORR'], $observation['NRPRODCOMVENOBS'],
                                                $observation['QTPRODCOMVENOBS']);
        }
    }
    */
    
    public function insertProduct ($product) {
        
        $this->factory->insertProduct($product);
        
        if(!empty($product["ITEM_OBSERVATION"])) {
            foreach($product["ITEM_OBSERVATION"] as $obsItem) {
                $this->factory->insertObservation($obsItem);
            }
        }
        
    }
    
}