import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable, of } from 'rxjs';
import { catchError, map, tap } from 'rxjs/operators';

import { Client } from './models/Client';

@Injectable({
  providedIn: 'root'
})

export class ClientsService {
  /*teste base url*/
  //private baseUrl = 'https://tablet.teknisa.com/brunapinto/odhen-ubereats/webhook/index.php';
  //private baseUrl = 'http://ketersonflores1.zeedhi.com/workfolder/anota-ai/webhook/backend/webhook/index.php';
  //private baseUrl = ''//https://uber-eats.teknisa.cloud/backend_ubereats/index.php';
  private baseUrl = 'https://anota-ai.teknisa.cloud/webhook/index.php';
  
  private httpOptions = {
    headers: new HttpHeaders({ 'Content-Type' : 'application/json' })
  };

  constructor( private http: HttpClient ) { 
  }
  
  getCustomers(): Observable<Client[]> {
    return this.http.get<any>(`${this.baseUrl}/getClientList`).pipe(
      tap(_ => console.log("Fetched Customers")),
      map(res => res.data as Client[]),
      catchError(err => this.handleError(err))
    );
  }

  addNewToken ( data ): Observable<any> {
    return this.http.post(`${this.baseUrl}/addToken`,data).pipe(
      tap(_ => console.log(`add new token to the consumer id=${data.idCliente}`)),
      catchError(this.handleError<any>('add new token'))
    );
  }

  deleteToken( data ){
    return this.http.post(`${this.baseUrl}/deleteToken`,data).pipe(
      catchError(this.handleError<any>('delete Token'))
    );
  }

  editToken( data ) {
    return this.http.post(`${this.baseUrl}/editToken`,data).pipe(
      catchError(this.handleError<any>('edit Token'))
    );
  }

  editClientInfo( data ) {
    return this.http.post(`${this.baseUrl}/editClient`,data).pipe(
      catchError(this.handleError<any>('edit Client name'))
    );
  }

  editClienteUrl( data ) {
    return this.http.post(`${this.baseUrl}/editClient`,data).pipe(
      catchError(this.handleError<any>('edit Client name'))
    );
  }

  addCustomer( data ){
    return this.http.post(`${this.baseUrl}/addClient`,data).pipe(
      catchError(this.handleError<any>('add Client'))
    );
  }

  deleteClient( data ) {
    return this.http.post(`${this.baseUrl}/deleteClient`,data).pipe(
      catchError(this.handleError<any>('delete Client'))
    );
  }

  handleError<T> ( operation = 'operation', result?: T ) {
    return (error: any): Observable<T> => {

      // TODO: send the error to remote logging infrastructure
      console.error(error); // log to console instead
  
      // TODO: better job of transforming error for user consumption
      console.log(`${operation} failed: ${error.message}`);
  
      // Let the app keep running by returning an empty result.
      return of(result as T);
    };
  }

  uploadDataBase( formData ) {
    return this.http.post<any>(`${this.baseUrl}/dataBase`, formData, {  
      reportProgress: true,  
      observe: 'events'  
    }); 
  }
}
